function  [mask_labeled, varargout] = create_subregions(mask, dgt, idgt, ...,
    dgt_params, signal_params, tol, fig_dir)

%% [mask_labeled, varargout] = create_subregions(mask, dgt, idgt, dgt_params, signal_params, tol, fig_dir)
%  This function splits the whole region $\Omega$ in such sub-regions.[1]
%
% Inputs:
%      - mask: original mask
%      - dgt, idgt : Gabor transform and its inverse operator
%      - dgt_params : Discrete Gabor Transform parameters(hop, nbins,win, ect..)
%      -signal_params: Signals parameters(sig_len, fs)
%      - tol: tolerance .see [1]
%      - fig_dir :  Figures directory
%
% Outputs:
%     - mask_labeled: mask with P regions
%     -pq_norms_val: norms between two Gabor multipliers
%
%
% Reference:
% [1]Time-frequency fading algorithms based on Gabor multipliers,2020.
%
%
% Author: Marina KREME
%%
mask = boolean(mask);
[mask_labeled, n_labels] = bwlabel(mask);
sig_len = signal_params.sig_len;
pq_norms_val = get_pq_norms(sig_len,dgt,idgt,mask_labeled);

%%
figure;
plot_mask(mask_labeled, dgt_params.hop, dgt_params.nbins, signal_params.fs)
title('Initial subregions')
saveas(gcf,fullfile(fig_dir, 'initial_subregions.pdf'));

figure;
imagesc(real(log10(pq_norms_val+pq_norms_val')))
ylabel('p')
xlabel('q')
colorbar()
title('Initial norms of Gabor multiplier composition')
saveas(gcf,fullfile(fig_dir, 'initial_norms.pdf'));

n_labels_max = n_labels;

%%

while max(pq_norms_val(:))>tol
    %Merge each pair (p, q), q < p, such that pq_norms[p, q] > tol
    to_be_updated = boolean(zeros(n_labels,1));
    
    while max(pq_norms_val(:))>tol
        [i_p, i_q] =  find(pq_norms_val == max(pq_norms_val(:)));
        
        
        mask = mask_labeled;
        [mask_labeled, pq_norms_val] = merge_subregions(mask, pq_norms_val, i_p, i_q);
        
        to_be_updated(i_q)= true;
        to_be_updated(i_p) = to_be_updated(end);
        to_be_updated = to_be_updated(1:end-1);
        n_labels =n_labels- 1;
    end
    
    for i_p =1:n_labels
        if to_be_updated(i_p)==true
            pq_norms_val = update_pq_norms(mask_labeled, pq_norms_val, i_p, signal_params, dgt, idgt);
        end
    end
    
    
    
    
    
    figure;
    plot_mask(mask_labeled, dgt_params.hop, dgt_params.nbins, signal_params.fs)
    title('subregions ')
    saveas(gcf,fullfile(fig_dir, ['subregions_i_', num2str(n_labels_max-n_labels),'.pdf']));
    
    
    figure;
    imagesc(real(log10(pq_norms_val+pq_norms_val')))
    ylabel('p-1')
    xlabel('q-1')
    colorbar()
    title('norms of Gabor multiplier composition')
    saveas(gcf,fullfile(fig_dir, ['norms_i', num2str(n_labels_max-n_labels),'.pdf']));
    
    
end


%%



figure;
plot_mask(mask_labeled, dgt_params.hop, dgt_params.nbins, signal_params.fs)
title('Final subregions')
saveas(gcf,fullfile(fig_dir,'final_subregions.pdf'));


figure;
imagesc(real(log10(pq_norms_val+pq_norms_val')))
ylabel('p-1')
xlabel('q-1')
colorbar()
title('Final norms of Gabor multiplier composition')
saveas(gcf,fullfile(fig_dir, 'final_norms.pdf'));

varargout{1} = pq_norms_val;

end




